
package org.mule.modules.twiml.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.factories.MessageProcessorChainFactoryBean;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.twiml.config.SmsMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.w3c.dom.Element;

public class SmsDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public SmsDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(SmsMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        String actionCallbackFlowName = getAttributeValue(element, "action-flow-ref");
        if (actionCallbackFlowName!= null) {
            builder.addPropertyValue("actionCallbackFlow", new RuntimeBeanReference(actionCallbackFlowName));
        }
        if ((element.getAttribute("from")!= null)&&(!StringUtils.isBlank(element.getAttribute("from")))) {
            builder.addPropertyValue("from", element.getAttribute("from"));
        }
        if ((element.getAttribute("to")!= null)&&(!StringUtils.isBlank(element.getAttribute("to")))) {
            builder.addPropertyValue("to", element.getAttribute("to"));
        }
        String statusCallbackFlowName = getAttributeValue(element, "status-flow-ref");
        if (statusCallbackFlowName!= null) {
            builder.addPropertyValue("statusCallbackFlow", new RuntimeBeanReference(statusCallbackFlowName));
        }
        if (element!= null) {
            String text = element.getAttribute("text");
            if ((text!= null)&&(!StringUtils.isBlank(text))) {
                builder.addPropertyValue("nestedProcessors", text);
            } else {
                BeanDefinitionBuilder nestedProcessorsBeanDefinitionBuilder = BeanDefinitionBuilder.rootBeanDefinition(MessageProcessorChainFactoryBean.class);
                BeanDefinition nestedProcessorsBeanDefinition = nestedProcessorsBeanDefinitionBuilder.getBeanDefinition();
                parserContent.getRegistry().registerBeanDefinition(generateChildBeanName(element), nestedProcessorsBeanDefinition);
                element.setAttribute("name", generateChildBeanName(element));
                nestedProcessorsBeanDefinitionBuilder.setSource(parserContent.extractSource(element));
                nestedProcessorsBeanDefinitionBuilder.setScope(BeanDefinition.SCOPE_SINGLETON);
                List nestedProcessorsList = parserContent.getDelegate().parseListElement(element, nestedProcessorsBeanDefinitionBuilder.getBeanDefinition());
                parserContent.getRegistry().removeBeanDefinition(generateChildBeanName(element));
                builder.addPropertyValue("nestedProcessors", nestedProcessorsList);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
            if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                propertyValues.addPropertyValue("messageProcessors", new ManagedList());
            }
            List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
            listMessageProcessors.add(definition);
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
